# Corporate Calendar Zope product
#
# By Peter Bengtsson, peter@fry-it.com,
# Fry-IT Ltd. London, 2004
# www.fry-it.com
# 
# See: zope.org/Members/malikz/CorpCalendar
#

import re
from pprint import pprint

from Globals import Persistent, InitializeClass, package_home, DTMLFile
from OFS import SimpleItem, ObjectManager, Folder
from DocumentTemplate import sequence
from AccessControl import ClassSecurityInfo
from zLOG import LOG, ERROR, INFO, PROBLEM, WARNING
from DateTime import DateTime

# Product
from TemplateAdder import addTemplates2Class, CTP
import Utils
from Constants import *


#----------------------------------------------------------------------------

manage_addCorpCalendarForm = DTMLFile('dtml/addCalendar', globals())

def manage_addCorpCalendar(dispatcher, id, title='', REQUEST=None):
    """ add CorpCalendar instance via the web """
    dest = dispatcher.Destination()
    instance = CorpCalendar(id, title)
    dest._setObject(id, instance)
    self = dest._getOb(id)
    
    if REQUEST is not None:
        # whereto next?
        redirect = REQUEST.RESPONSE.redirect
        redirect(REQUEST.URL1+'/manage_workspace')


#----------------------------------------------------------------------------

OK_FORMATS = ('plaintext','structuredtext','html')
CORP_CALENDAR_METATYPE = "Corporate Calendar"
CORP_EVENT_METATYPE = "Corporate Event"

class CorpCalendar(Folder.Folder, Persistent):

    meta_type = CORP_CALENDAR_METATYPE

    security=ClassSecurityInfo()
    def __init__(self, id, title=''):
        """ init """
        self.id = id
        self.title = title
        self.ok_formats = OK_FORMATS
        self._user_variables = {}

    def getTitle(self):
        """ return title """
        return self.title

    def getRoot(self):
        """ return root object instance """
        mtype = CORP_CALENDAR_METATYPE
        r = self
        c =0
        while r.meta_type != mtype:
            r = r.aq_parent
            c+=1
            if c>20:
                return None
        return r

    def getRootURL(self):
        """ return absolute_url of getRoot() """
        return self.getRoot().absolute_url()


    security.declareProtected(VMS, 'manage_addEvent')
    def manage_addEvent(self, description, format, submitter,
                 start_date, end_date,
                 REQUEST=None):
        """ add event object """
        submitter = submitter.strip()
        description = description.strip()
        if not description:
            raise "NoDescription", "Description can't be empty"
        format = format.lower().strip()
        if format not in self.ok_formats:
            format = self.ok_formats[0]
        if Utils.same_type(start_date, 's'):
            start_date = DateTime(start_date)
        if Utils.same_type(end_date, 's'):
            end_date = DateTime(end_date)

        if end_date < start_date:
            raise "InvalidDates", "End date before start date"

        c=1
        id = origid = DateTime().strftime('%a-%d-%b-%y')
        while hasattr(self, id):
            id = origid + "-%s"%c
            c += 1

        adder = self.manage_addEventObject
        object = adder(id, description, format, submitter,
                       start_date, end_date)

        self.rememberUserVariable('format', format)
        self.rememberUserVariable('submitter', submitter)

        if REQUEST is not None:
            url = self.absolute_url()
            params = {}
            if REQUEST.get('date-calendar'):
                params['date-calendar'] = REQUEST.get('date-calendar')

            if REQUEST.get('mode-calendar'):
                params['mode-calendar'] = REQUEST.get('mode-calendar')

            url = Utils.AddParam2URL(url, params)
            REQUEST.RESPONSE.redirect(url)

    security.declareProtected(VMS, 'manage_addEventObject')
    def manage_addEventObject(self, id, description, format, submitter,
                        start_date, end_date):
        """ create object """
        instance = CorpEvent(id, description, format, submitter,
                             start_date, end_date)
        self._setObject(id, instance)
        object = self._getOb(id)
        object._preCacheDates()
        return object
    

    def rememberUserVariable(self, key, value):
        """ set in stone """
        cookiekey = '__corpcal_%s'%key.replace(' ','')
        user = self.REQUEST.AUTHENTICATED_USER
        username = user.name
        # cookie
        then = DateTime()+300
        then = then.rfc822()
        response = self.REQUEST.RESPONSE
        response.setCookie(cookiekey, value, expires=then, path='/')
        
        if username.lower().replace(' ','') != 'anonymoususer':
            _user_vs = self._user_variables
            if not self._user_variables.has_key(username):
                _user_vs[username] = {}
                         
            _user_vs[key] = value
            self._user_variables = _user_vs

    def getUserVariable(self, key, default=None):
        """ get from stone """
        cookiekey = '__corpcal_%s'%key.replace(' ','')
        user = self.REQUEST.AUTHENTICATED_USER
        username = user.name
        # cookie
        if self.REQUEST.cookies.has_key(cookiekey):
            return self.REQUEST.cookies.get(cookiekey)
        
        
        if username.lower().replace(' ','') != 'anonymoususer':
            if self._user_variables.has_key(username):
                if self._user_variables[username].has_key(key):
                    return self._user_variables[username][key]
                
        return default


    security.declareProtected('View', 'getAllEvents')
    def getAllEvents(self):
        """ return all event objects """
        return self.objectValues(CORP_EVENT_METATYPE)

    security.declareProtected('View', 'filterEvents')
    def filterEvents(self, date, events=None):
        """ filter on matching dates """
        if events is None:
            events = self.getAllEvents()
        ok = []
        for event in events:
            if event.onDate(date):
                ok.append(event)

        return ok



    decl = security.declareProtected
    decl('View', 'index_html')
    decl(VMS, 'addEvent')
    decl('View','standard_html_header')
    decl('View','standard_html_footer')
    decl('View','styles.css')
    
dtmls = ('dtml/index_html',
         'dtml/addEvent',
         'dtml/standard_html_header',
         'dtml/standard_html_footer',
         {'f':'dtml/styles.css','optimize':OPTIMIZE and 'css'},
         )

addTemplates2Class(CorpCalendar, dtmls, extension='dtml')

InitializeClass(CorpCalendar)

#----------------------------------------------------------------------------

manage_addCorpEventForm = DTMLFile('dtml/NotImplemented', globals())

def manage_addCorpEvent(dispatcher, id, *args, **kws):
    """ bad """
    raise "NotImplemented", "Use View part of site instead"


class CorpEvent(SimpleItem.SimpleItem, Persistent):
    """ Event class """

    meta_type = CORP_EVENT_METATYPE

    _properties=({'id':'description',   'type': 'text',   'mode':'w'},
                 {'id':'format',        'type': 'string', 'mode':'w'},
                 {'id':'start_date',    'type': 'date',   'mode':'r'},
                 {'id':'end_date',      'type': 'date',   'mode':'r'},
                 )

    security=ClassSecurityInfo()
    def __init__(self, id, description, format, submitter,
                 start_date, end_date):
        """ init """
        self.id = id
        self.description = description
        self.format = format
        self.submitter = submitter
        self.start_date = start_date
        self.end_date = end_date
        self._datestring_list = []
	
    def getId(self):
	""" return id """
	return self.id

    def getTitle(self, max=30):
        """ return parts of the description """
        if len(self.description) > max:
            return self.description[:max]+'...'
        else:
            return self.description

    def getSubmitter(self):
        """ return self.submitter """
        return self.submitter

    def getDescription(self):
        """ return description """
        return self.description

    def showDescription(self):
        """ return description nicely """
        return Utils.ShowDescription(self.getDescription(),
                                     self.getFormat())

    def getFormat(self):
        """ return format """
        return self.format

    def getStartDate(self):
        """ return start_date """
        return self.start_date

    def getEndDate(self):
        """ return self.end_date """
        return self.end_date
    
    def showDateDifference(self):
	""" return a string of the difference between two dates """
	return Utils.timeSince(self.getStartDate(), self.getEndDate())

    def onDate(self, dateobj):
        """ true if this date is there """
        if Utils.same_type(dateobj, 's'):
            dateobj = DateTime(dateobj)

        return dateobj.strftime('%d%m%Y') in self._datestring_list

    def _preCacheDates(self):
        """ make a list of strings of the dates for this start_date
        and end_date """
        if int(self.end_date - self.start_date) ==0: # same day
            list = [self.start_date.strftime('%d%m%Y')]
        else:
            list = []
            for i in range(int(self.end_date - self.start_date)):
                _d = self.start_date + i
                list.append(_d.strftime('%d%m%Y'))
                
        self._datestring_list = list


    security.declareProtected(VMS, 'manage_editEvent')
    def manage_editEvent(self, description, format, submitter,
                         start_date, end_date, REQUEST=None):
        """ edit event object """
        description = description.strip()
        submitter = submitter.strip()
        if not description:
            raise "NoDescription", "Description can't be empty"
        format = format.strip().lower()
        if format not in self.ok_formats:
            format = self.ok_formats[0]

        if Utils.same_type(start_date,'s'):
            start_date = DateTime(start_date)
        if Utils.same_type(end_date, 's'):
            end_date = DateTime(end_date)

        if end_date < start_date:
            raise "InvalidDates", "End date before start date"

        # go ahead and change
        self.description = description
        self.submitter = submitter
        self.format = format
        self.start_date = start_date
        self.end_date = end_date

        self.rememberUserVariable('submitter', submitter)
        self.rememberUserVariable('format', format) 

        self._preCacheDates()

        if REQUEST is not None:
            url = self.getRootURL()
            params = {}
            if REQUEST.get('date-calendar'):
                params['date-calendar'] = REQUEST.get('date-calendar')

            if REQUEST.get('mode-calendar'):
                params['mode-calendar'] = REQUEST.get('mode-calendar')

            url = Utils.AddParam2URL(url, params)
            REQUEST.RESPONSE.redirect(url)

	    
    def getEventURL(self):
	""" return absolute_url() but with some other possible 
	variables. """
	request = self.REQUEST
	url = self.absolute_url()
	params = {}
	if request.get('date-calendar'):
	    params['date-calendar'] = request.get('date-calendar')

	if request.get('mode-calendar'):
	    params['mode-calendar'] = request.get('mode-calendar')

	url = Utils.AddParam2URL(url, params)
	
	return url
	
	

    decl = security.declareProtected
    decl(VMS, 'index_html')
    decl(VMS, 'editEvent')

dtmls = (('dtml/view_event','index_html'),
         ('dtml/editEvent','edit'),
         )
addTemplates2Class(CorpEvent, dtmls, extension='dtml')
    
InitializeClass(CorpEvent)    
